//
//  ViewController.m
//  WiMaster_test3
//
//  Created by 石井 孝佳 on 13/08/15.
//  Copyright (c) 2013年 石井 孝佳. All rights reserved.
//

#import "ViewController.h"
#import "WiFiConnect.h"

#define READ_BUTTON 0

@interface ViewController ()

@end

@implementation ViewController{
    dispatch_semaphore_t _semaphore;
    NSInputStream* _readStream;
    NSOutputStream* _writeStream;
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //---_sliderの生成---
    //    _slider=[[UISlider alloc] init];
    _slider=[UISlider new];
    [_slider setFrame:CGRectMake(0,0,300,50)];  //位置と大きさ設定
    [_slider setMinimumValue:0];                //サーボ位置最低
    [_slider setMaximumValue:254];              //サーボ位置最高
    [_slider setValue:127];                     //初期位置はサーボ中心位置
    [_slider addTarget:self action:@selector(onSliderChanged:) forControlEvents:UIControlEventValueChanged];    //スライダイベント登録
    [self.view addSubview:_slider];             //画面に追加
    
    //---センサー値読み取りボタン生成---
    _readButton=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [_readButton setFrame:CGRectMake(0,80,250,40)];  //位置と大きさ設定
    [_readButton setTitle:@"センサー読み取り" forState:UIControlStateNormal];
    [_readButton setTag:READ_BUTTON];           //ボタン識別タグ
    [_readButton addTarget:self action:@selector(onButtonClick:) forControlEvents:UIControlEventTouchUpInside];             //ボタンクリックイベント登録
    [self.view addSubview:_readButton];
    
    //---センサー値結果のテキストフィールド生成---
    _textField=[[UITextField alloc] init];
    [_textField setFrame:CGRectMake(0,150,100,30)];  //位置と大きさ設定
    [_textField setText:@"result"];
    [_textField setBackgroundColor:[UIColor whiteColor]];
    [_textField setBorderStyle:UITextBorderStyleRoundedRect];
    [self.view addSubview:_textField];
    
    //---WiMasterに接続---
    [self connect];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

//////////////////////////////////////////////////////////////
//  WiMasterと接続
//////////////////////////////////////////////////////////////
-(void)connect
{
    _conn = [[WiFiConnect alloc] initWithHostName:@"1.2.3.4" port:2000 timeout:30];
    _conn.delegate = self; //受信イベント受け取りの為、デリゲートをセット
    if([_conn openSocket]){
        NSLog(@"openSocket success");
    }else{
        NSLog(@"openSocket Error");
    }
}


//////////////////////////////////////////////////////////////
//  デストラクタ
//////////////////////////////////////////////////////////////
-(void)dealloc
{
    [_conn closeSocket];
}


//////////////////////////////////////////////////////////////
//  ボタンクリックイベント
//////////////////////////////////////////////////////////////
-(IBAction)onButtonClick:(UIButton*)sender{
    //読み取りボタンクリック時処理
    if(sender.tag==READ_BUTTON){
        //センサー読み取り命令送信
        [self sendSensoeReadCommand];           
        //受信データは受信イベントで受け取る
    }
}

//////////////////////////////////////////////////////////////
//  データ受信イベント（WiFiConnectより）
//////////////////////////////////////////////////////////////
-(void)WiFiDataReceived:(NSMutableData *)data{
    NSLog(@"data=%@",data);
    //#3のセンサー値を調べる（テスト回路では#3に距離センサーがつながっている）
    int result = [self checkSensorValue:data sensorNumber:3];
    //画面に表示
    [_textField setText:[NSString stringWithFormat:@"%d",result]];
    
}

//返信dataから指定のサーボ番号の結果値を返す
-(int)checkSensorValue:(NSMutableData *)data sensorNumber:(int)sNum{
    int result = -1;
    //MutableDataをbyte配列に変換
    int len = [data length];
    Byte rsvData[len];
    [data getBytes:&rsvData length:len];
    //指定番号のセンサー値を抽出（AGB65-ADCからの返信を解析）
    //ヘッダ長が4バイトなので最後から4手前までのデータを確認
    for(int i=0; i<len-4; i++){
        //返信フォーマット [255][ID(120-127)][バイト長(17)][命令(1)][P1結果][P2結果]...[P16結果]
        if(rsvData[i]==255 && rsvData[i+1]==120 && rsvData[i+2]==17
           && rsvData[i+3]==1){
            result=rsvData[i+3+sNum];   //sNumで指定したサーボ番号の結果値
            break;
        }
    }
    return result;
}


//////////////////////////////////////////////////////////////
//  スライダー変化イベント
//////////////////////////////////////////////////////////////
-(IBAction)onSliderChanged:(UISlider*)sender{
    NSLog(@"pos=%f\n", sender.value);
    [self sendServoPosition:0 pos:sender.value psd:1];
}


//////////////////////////////////////////////////////////////
//  サーボ位置を送信
//////////////////////////////////////////////////////////////
-(void) sendServoPosition:(int)num pos:(float)pos psd:(int)spd
{
    uint8_t data[10];
    data[0]=255;        //sync byte
    data[1]=3;          //board id(AGB65-RSC2 default id)
    data[2]=4;          //data length
    data[3]=2;          //com(single servo drive command)
    data[4]=num;        //servo number
    data[5]=pos;        //position
    data[6]=spd;        //speed
    [_conn writeData:data length:7];
}

//////////////////////////////////////////////////////////////
//  センサー読み取り命令を送信
//////////////////////////////////////////////////////////////
-(void) sendSensoeReadCommand
{
    uint8_t data[10];
    data[0]=255;        //sync byte
    data[1]=120;        //board id(AGB65-ADC default id)
    data[2]=1;          //data length
    data[3]=1;          //com(all sensor read 8bit command)
    [_conn writeData:data length:4];
}


@end
